@@ -7,9 +7,11 @@
 // -----------------------------------------------------------------------------
 // All defined below
 
-static void compute_window_sizes(int*, SEXP, int);
-static void compute_window_starts(int*, int*, int);
-static void compute_window_stops(int*, int*, int*, int);
+static void fill_window_info(int* window_sizes,
+                             int* window_starts,
+                             int* window_stops,
+                             SEXP window_indices,
+                             int size);
 
 static struct window_info new_window_info(int*, int*, int);
 static struct index_info new_index_info(SEXP);
@@ -36,12 +38,14 @@ SEXP slide_index_common_impl(SEXP x,
                              SEXP indices,
                              SEXP type_,
                              SEXP constrain_,
+                             SEXP atomic_,
                              SEXP size_,
                              SEXP complete_) {
   int n_prot = 0;
 
   int type = r_scalar_int_get(type_);
   bool constrain = r_scalar_lgl_get(constrain_);
+  bool atomic = r_scalar_lgl_get(atomic_);
   int size = r_scalar_int_get(size_);
   bool complete = r_scalar_lgl_get(complete_);
 
@@ -54,9 +58,7 @@ SEXP slide_index_common_impl(SEXP x,
   int* window_starts = (int*) R_alloc(index.size, sizeof(int));
   int* window_stops = (int*) R_alloc(index.size, sizeof(int));
 
-  compute_window_sizes(window_sizes, indices, index.size);
-  compute_window_starts(window_starts, window_sizes, index.size);
-  compute_window_stops(window_stops, window_sizes, window_starts, index.size);
+  fill_window_info(window_sizes, window_starts, window_stops, indices, index.size);
 
   struct window_info window = new_window_info(window_starts, window_stops, index.size);
   PROTECT_WINDOW_INFO(&window, &n_prot);
@@ -76,6 +78,13 @@ SEXP slide_index_common_impl(SEXP x,
   REPROTECT(out, out_prot_idx);
   ++n_prot;
 
+  // Initialize with `NA`, not `NULL`, for size stability when auto-simplifying
+  if (atomic && !constrain) {
+    for (R_len_t i = 0; i < size; ++i) {
+      SET_VECTOR_ELT(out, i, slider_shared_na_lgl);
+    }
+  }
+
   for (int i = min_iteration; i < max_iteration; ++i) {
     if (i % 1024 == 0) {
       R_CheckUserInterrupt();
@@ -93,15 +102,13 @@ SEXP slide_index_common_impl(SEXP x,
     SEXP out_index = VECTOR_ELT(indices, i);
     int out_index_size = vec_size(out_index);
 
+    if (atomic && vec_size(elt) != 1) {
+      stop_not_all_size_one(i + 1, vec_size(elt));
+    }
+
     if (constrain) {
       elt = PROTECT(vec_cast(elt, ptype));
 
-      R_len_t elt_size = vec_size(elt);
-
-      if (elt_size != 1) {
-        stop_not_all_size_one(i + 1, elt_size);
-      }
-
       // Must always PROTECT() to avoid rchk note, see #58
       if (out_index_size != 1) {
         elt = vec_recycle(elt, out_index_size);
@@ -144,11 +151,13 @@ SEXP hop_index_common_impl(SEXP x,
                            SEXP window_indices,
                            SEXP type_,
                            SEXP constrain_,
+                           SEXP atomic_,
                            SEXP size_) {
   int n_prot = 0;
 
   int type = r_scalar_int_get(type_);
   bool constrain = r_scalar_lgl_get(constrain_);
+  bool atomic = r_scalar_lgl_get(atomic_);
   int size = r_scalar_int_get(size_);
 
   int force = compute_force(type);
@@ -160,9 +169,7 @@ SEXP hop_index_common_impl(SEXP x,
   int* window_starts = (int*) R_alloc(index.size, sizeof(int));
   int* window_stops = (int*) R_alloc(index.size, sizeof(int));
 
-  compute_window_sizes(window_sizes, window_indices, index.size);
-  compute_window_starts(window_starts, window_sizes, index.size);
-  compute_window_stops(window_stops, window_sizes, window_starts, index.size);
+  fill_window_info(window_sizes, window_starts, window_stops, window_indices, index.size);
 
   struct window_info window = new_window_info(window_starts, window_stops, index.size);
   PROTECT_WINDOW_INFO(&window, &n_prot);
@@ -179,6 +186,13 @@ SEXP hop_index_common_impl(SEXP x,
   REPROTECT(out, out_prot_idx);
   ++n_prot;
 
+  // Initialize with `NA`, not `NULL`, for size stability when auto-simplifying
+  if (atomic && !constrain) {
+    for (R_len_t i = 0; i < size; ++i) {
+      SET_VECTOR_ELT(out, i, slider_shared_na_lgl);
+    }
+  }
+
   // 1 based index for `vec_assign()`
   SEXP out_index;
   int* p_out_index;
@@ -202,17 +216,15 @@ SEXP hop_index_common_impl(SEXP x,
     SEXP elt = PROTECT(Rf_eval(f_call, env));
 #endif
 
+    if (atomic && vec_size(elt) != 1) {
+      stop_not_all_size_one(i + 1, vec_size(elt));
+    }
+
     if (constrain) {
-      elt = PROTECT(vec_cast(elt, ptype));
-
-      R_len_t elt_size = vec_size(elt);
-
-      if (elt_size != 1) {
-        stop_not_all_size_one(i + 1, elt_size);
-      }
-
       *p_out_index = i + 1;
 
+      elt = PROTECT(vec_cast(elt, ptype));
+
       out = vec_proxy_assign(out, out_index, elt);
       REPROTECT(out, out_prot_idx);
 
@@ -227,9 +239,6 @@ SEXP hop_index_common_impl(SEXP x,
   out = vec_restore(out, ptype);
   REPROTECT(out, out_prot_idx);
 
-  out = copy_names(out, x, type);
-  REPROTECT(out, out_prot_idx);
-
   UNPROTECT(n_prot);
   return out;
 }
@@ -347,36 +356,21 @@ static int iteration_max_adjustment(struct index_info index, SEXP range, int siz
 
 // -----------------------------------------------------------------------------
 
-// map_int(x, vec_size)
-static void compute_window_sizes(int* window_sizes,
-                                 SEXP window_indices,
-                                 int size) {
+static void fill_window_info(int* window_sizes,
+                             int* window_starts,
+                             int* window_stops,
+                             SEXP window_indices,
+                             int size) {
+  R_len_t window_start = 0;
+
   for (int i = 0; i < size; ++i) {
-    window_sizes[i] = Rf_length(VECTOR_ELT(window_indices, i));
-  }
-}
+    R_len_t window_size = Rf_length(VECTOR_ELT(window_indices, i));
 
-static void compute_window_starts(int* window_starts,
-                                  int* window_sizes,
-                                  int size) {
-  // First start is always 0
-  window_starts[0] = 0;
+    window_sizes[i] = window_size;
+    window_starts[i] = window_start;
+    window_stops[i] = window_start + window_size - 1;
 
-  int sum = 0;
-
-  // Then we do a cumsum() to get the rest of the starts
-  for (int i = 1; i < size; ++i) {
-    sum += window_sizes[i - 1];
-    window_starts[i] = sum;
-  }
-}
-
-static void compute_window_stops(int* window_stops,
-                                 int* window_sizes,
-                                 int* window_starts,
-                                 int size) {
-  for (int i = 0; i < size; ++i) {
-    window_stops[i] = window_starts[i] + window_sizes[i] - 1;
+    window_start += window_size;
   }
 }
 
@@ -385,46 +379,50 @@ static void compute_window_stops(int* window_stops,
 // update the current start/stop position
 
 static int locate_window_starts_pos(struct index_info* index, struct range_info range, int pos) {
-  if (range.start_unbounded || index->compare_lt(range.starts, pos, index->data, 0)) {
-    if (range.stop_unbounded) {
-      return 0;
-    }
-
-    if (index->compare_lt(range.stops, pos, index->data, 0)) {
-      return -1;
-    }
-
+  // Pin to the start
+  if (range.start_unbounded) {
     return 0;
   }
 
-  while(index->compare_lt(index->data, index->current_start_pos, range.starts, pos)) {
-    if (index->current_start_pos == index->last_pos) {
-      return index->current_start_pos;
-    }
+  // Past the end? Signal OOB with `last_pos + 1`.
+  // This also handles size zero `.i` with `.starts` / `.stops` that have size.
+  // Current pos will be 0, but `last_pos` will be -1.
+  if (index->current_start_pos > index->last_pos) {
+    return index->last_pos + 1;
+  }
+
+  while (index->compare_lt(index->data, index->current_start_pos, range.starts, pos)) {
     ++index->current_start_pos;
+
+    // Past the end? Signal OOB with `last_pos + 1`.
+    if (index->current_start_pos > index->last_pos) {
+      return index->last_pos + 1;
+    }
   }
 
   return index->current_start_pos;
 }
 
 static int locate_window_stops_pos(struct index_info* index, struct range_info range, int pos) {
-  if (range.stop_unbounded || index->compare_gt(range.stops, pos, index->data, index->last_pos)) {
-    if (range.start_unbounded) {
-      return index->last_pos;
-    }
-
-    if (index->compare_gt(range.starts, pos, index->data, index->last_pos)) {
-      return -1;
-    }
-
+  // Pin to the end
+  if (range.stop_unbounded) {
     return index->last_pos;
   }
 
-  while(index->compare_lte(index->data, index->current_stop_pos, range.stops, pos)) {
-    if (index->current_stop_pos == index->last_pos) {
-      return index->current_stop_pos;
-    }
+  // Past the end? Pin to end.
+  // This also handles size zero `.i` with `.starts` / `.stops` that have size.
+  // Current pos will be 0, but `last_pos` will be -1.
+  if (index->current_stop_pos > index->last_pos) {
+    return index->last_pos;
+  }
+
+  while (index->compare_lte(index->data, index->current_stop_pos, range.stops, pos)) {
     ++index->current_stop_pos;
+
+    // Past the end? Pin to end.
+    if (index->current_stop_pos > index->last_pos) {
+      return index->last_pos;
+    }
   }
 
   return index->current_stop_pos - 1;
@@ -439,16 +437,6 @@ static void increment_window(struct window_info window,
   int starts_pos = locate_window_starts_pos(index, range, pos);
   int stops_pos = locate_window_stops_pos(index, range, pos);
 
-  // This is our signal that we are outside the range of `i`. For example,
-  // i = 1:2, but we are trying to index [start = 3, stop = 4]. In these cases
-  // there is "no data" in that range, so we pass a size 0 slice of `x` to `f`
-  if (starts_pos == -1 || stops_pos == -1) {
-    init_compact_seq(window.p_seq_val, 0, 0, true);
-    return;
-  }
-
-  // This can happen with an irregular index, and is a sign of the full window
-  // being between two index points and means we select nothing
   if (stops_pos < starts_pos) {
     init_compact_seq(window.p_seq_val, 0, 0, true);
     return;
