@@ -10,15 +10,15 @@ test_that("size of each `.f` result must be 1", {
 
 test_that("inner type is allowed to be different", {
   expect_equal(
-    hop_index_vec(1:2, 1:2, 1:2, 1:2, ~if (.x == 1L) {1} else {"hi"}, .ptype = list()),
+    hop_index_vec(1:2, 1:2, 1:2, 1:2, ~if (.x == 1L) {list(1)} else {list("hi")}, .ptype = list()),
     list(1, "hi")
   )
 })
 
 test_that("inner type can be restricted with list_of", {
   expect_error(
-    hop_index_vec(1:2, 1:2, 1:2, 1:2, ~if (.x == 1L) {1} else {"hi"}, .ptype = list_of(.ptype = double())),
-    class = "vctrs_error_cast_lossy"
+    hop_index_vec(1:2, 1:2, 1:2, 1:2, ~if (.x == 1L) {list_of(1)} else {list_of("hi")}, .ptype = list_of(.ptype = double())),
+    class = "vctrs_error_incompatible_type"
   )
 })
 
@@ -28,7 +28,6 @@ test_that("inner type can be restricted with list_of", {
 test_that(".ptype is respected", {
   expect_equal(hop_index_vec(1, 1, 1, 1, ~.x), 1)
   expect_equal(hop_index_vec(1, 1, 1, 1, ~.x, .ptype = int()), 1L)
-  expect_equal(hop_index_vec(1, 1, 1, 1, ~.x, .ptype = new_date()), as.Date("1970-01-02"))
   expect_error(hop_index_vec(1, 1, 1, 1, ~.x + .5, .ptype = integer()), class = "vctrs_error_cast_lossy")
 })
 
@@ -51,13 +50,21 @@ test_that("`.ptype = NULL` validates that element lengths are 1", {
     hop_index_vec(1:2, 1:2, 1:2, 1:2, ~if(.x == 1L) {1:2} else {1}, .ptype = NULL),
     "In iteration 1, the result of `.f` had size 2, not 1."
   )
+  expect_error(
+    hop_index_vec(1:2, 1:2, 1:2, 1:2, ~if(.x == 1L) {NULL} else {2}, .ptype = NULL),
+    "In iteration 1, the result of `.f` had size 0, not 1."
+  )
 })
 
-test_that("`.ptype = NULL` returns `NULL` with size 0 `.starts` / `.stops`", {
-  expect_equal(
-    hop_index_vec(integer(), integer(), integer(), integer(), ~.x, .ptype = NULL),
+test_that("size 0 `.starts` / `.stops` returns size 0 `.ptype`", {
+  expect_identical(
+    hop_index_vec(1:5, 1:5, integer(), integer(), ~.x, .ptype = NULL),
     NULL
   )
+  expect_identical(
+    hop_index_vec(1:5, 1:5, integer(), integer(), ~.x, .ptype = double()),
+    double()
+  )
 })
 
 test_that("`.ptype = NULL` returns `NULL` with one size 0 and one size 1 starts / stops", {
@@ -94,47 +101,31 @@ test_that("can return a matrix and rowwise bind the results together", {
   )
 })
 
+test_that("`hop_index_vec()` falls back to `c()` method as required", {
+  local_c_foobar()
+
+  expect_identical(hop_index_vec(1:3, 1:3, 1:3, 1:3, ~foobar(.x), .ptype = foobar()), foobar(1:3))
+  expect_condition(hop_index_vec(1:3, 1:3, 1:3, 1:3, ~foobar(.x), .ptype = foobar()), class = "slider_c_foobar")
+
+  expect_identical(hop_index_vec(1:3, 1:3, 1:3, 1:3, ~foobar(.x)), foobar(1:3))
+  expect_condition(hop_index_vec(1:3, 1:3, 1:3, 1:3, ~foobar(.x)), class = "slider_c_foobar")
+})
+
 # ------------------------------------------------------------------------------
 # input names
 
 test_that("names exist on inner sliced elements", {
   names <- letters[1:5]
   x <- set_names(1:5, names)
-  exp <- set_names(as.list(names), names)
+  exp <- as.list(names)
   i <- vec_seq_along(x)
   expect_equal(hop_index_vec(x, i, i, i, ~list(names(.x))), exp)
 })
 
-test_that("names can be placed on atomics", {
+test_that("names are never placed on the output", {
   names <- letters[1:5]
   x <- set_names(1:5, names)
   i <- vec_seq_along(x)
-  expect_equal(names(hop_index_vec(x, i, i, i, ~.x)), names)
-  expect_equal(names(hop_index_vec(x, i, i, i, ~.x, .ptype = int())), names)
-  expect_equal(names(hop_index_vec(x, i, i, i, ~.x, .ptype = dbl())), names)
-})
-
-test_that("names are not placed on data frames rownames", {
-  names <- letters[1:2]
-  x <- set_names(1:2, names)
-  i <- vec_seq_along(x)
-  out <- hop_index_vec(x, i, i, i, ~data.frame(x = .x), .ptype = data.frame(x = int()))
-  expect_equal(rownames(out), c("1", "2"))
-})
-
-test_that("names can be placed on arrays", {
-  names <- letters[1:2]
-  x <- set_names(1:2, names)
-  i <- vec_seq_along(x)
-  out <- hop_index_vec(x, i, i, i, ~array(.x, c(1, 1)), .ptype = array(int(), dim = c(0, 1)))
-  expect_equal(rownames(out), names)
-})
-
-test_that("names can be placed correctly on proxied objects", {
-  names <- letters[1:2]
-  x <- set_names(1:2, names)
-  i <- vec_seq_along(x)
-  datetime_lt <- as.POSIXlt(new_datetime(0))
-  out <- hop_index_vec(x, i, i, i, ~datetime_lt, .ptype = datetime_lt)
-  expect_equal(names(out), names)
+  expect_null(names(hop_index_vec(x, i, i, i, ~.x)))
+  expect_null(names(hop_index_vec(x, i, i, i, ~.x, .ptype = int())))
 })
