@@ -6,19 +6,30 @@ test_that("size of each `.f` result must be 1", {
     slide_period_vec(1:2, new_date(1:2), "day", ~c(.x, 1)),
     "In iteration 1, the result of `.f` had size 2, not 1"
   )
+  expect_error(
+    slide_period_dbl(1:2, new_date(1:2), "day", ~c(.x, 1)),
+    "In iteration 1, the result of `.f` had size 2, not 1"
+  )
 })
 
 test_that("inner type is allowed to be different", {
   expect_equal(
-    slide_period_vec(1:2, new_date(1:2), "day", ~if (.x == 1L) {1} else {"hi"}, .ptype = list()),
+    slide_period_vec(1:2, new_date(1:2), "day", ~if (.x == 1L) {list(1)} else {list("hi")}, .ptype = list()),
     list(1, "hi")
   )
 })
 
 test_that("inner type can be restricted with list_of", {
   expect_error(
-    slide_period_vec(1:2, new_date(1:2), "day", ~if (.x == 1L) {1} else {"hi"}, .ptype = list_of(.ptype = double())),
-    class = "vctrs_error_cast_lossy"
+    slide_period_vec(1:2, new_date(1:2), "day", ~if (.x == 1L) {list_of(1)} else {list_of("hi")}, .ptype = list_of(.ptype = double())),
+    class = "vctrs_error_incompatible_type"
+  )
+})
+
+test_that("type can be restricted", {
+  expect_error(
+    slide_period_dbl(1:2, new_date(1:2), "day", ~if (.x == 1L) {1} else {"hi"}),
+    class = "vctrs_error_incompatible_type"
   )
 })
 
@@ -28,7 +39,6 @@ test_that("inner type can be restricted with list_of", {
 test_that(".ptype is respected", {
   expect_equal(slide_period_vec(1, new_date(0), "day", ~.x), 1)
   expect_equal(slide_period_vec(1, new_date(0), "day", ~.x, .ptype = int()), 1L)
-  expect_equal(slide_period_vec(1, new_date(0), "day", ~.x, .ptype = new_date()), as.Date("1970-01-02"))
   expect_error(slide_period_vec(1, new_date(0), "day", ~.x + .5, .ptype = integer()), class = "vctrs_error_cast_lossy")
 })
 
@@ -51,6 +61,10 @@ test_that("`.ptype = NULL` validates that element lengths are 1", {
     slide_period_vec(1:2, new_date(0:1), "day", ~if(.x == 1L) {1:2} else {1}, .ptype = NULL),
     "In iteration 1, the result of `.f` had size 2, not 1."
   )
+  expect_error(
+    slide_period_vec(1:2, new_date(0:1), "day", ~if(.x == 1L) {NULL} else {1}, .ptype = NULL),
+    "In iteration 1, the result of `.f` had size 0, not 1."
+  )
 })
 
 test_that("`.ptype = NULL` returns `NULL` with size 0 `.x`", {
@@ -72,7 +86,9 @@ test_that("with `.complete = TRUE`, `.ptype` is used to pad", {
     ),
     c(NA, 1, 1)
   )
+})
 
+test_that("with `.complete = TRUE`, padding is size stable (#93)", {
   expect_equal(
     slide_period_vec(
       1:3, new_date(1:3),
@@ -80,6 +96,20 @@ test_that("with `.complete = TRUE`, `.ptype` is used to pad", {
     ),
     new_date(c(NA, 0, 0))
   )
+  expect_equal(
+    slide_period_vec(
+      1:3, new_date(1:3),
+      "day", ~new_date(0), .after = 1, .complete = TRUE, .ptype = new_date()
+    ),
+    new_date(c(0, 0, NA))
+  )
+  expect_equal(
+    slide_period_vec(
+      1:3, new_date(1:3),
+      "day", ~new_date(0), .before = 1, .complete = TRUE, .ptype = NULL
+    ),
+    new_date(c(NA, 0, 0))
+  )
 })
 
 test_that("can return a matrix and rowwise bind the results together", {
@@ -90,6 +120,16 @@ test_that("can return a matrix and rowwise bind the results together", {
   )
 })
 
+test_that("`slide_period_vec()` falls back to `c()` method as required", {
+  local_c_foobar()
+
+  expect_identical(slide_period_vec(1:3, new_date(1:3), "day", ~foobar(.x), .ptype = foobar()), foobar(1:3))
+  expect_condition(slide_period_vec(1:3, new_date(1:3), "day", ~foobar(.x), .ptype = foobar()), class = "slider_c_foobar")
+
+  expect_identical(slide_period_vec(1:3, new_date(1:3), "day", ~foobar(.x)), foobar(1:3))
+  expect_condition(slide_period_vec(1:3, new_date(1:3), "day", ~foobar(.x)), class = "slider_c_foobar")
+})
+
 # ------------------------------------------------------------------------------
 # suffix tests
 
@@ -113,8 +153,8 @@ test_that("slide_period_chr() works", {
   expect_equal(slide_period_chr("x", new_date(0), "day", ~.x), "x")
 })
 
-test_that("slide_period_chr() can coerce", {
-  expect_equal(slide_period_chr(1, new_date(0), "day", ~.x), "1")
+test_that("slide_period_chr() cannot coerce", {
+  expect_error(slide_period_chr(1, new_date(0), "day", ~.x), class = "vctrs_error_incompatible_type")
 })
 
 test_that("slide_period_lgl() works", {
@@ -153,3 +193,20 @@ test_that("slide_period_dfc() works", {
     slide_dfc(x, ~data.frame(x = .x), .before = 1)
   )
 })
+
+# ------------------------------------------------------------------------------
+# input names
+
+test_that("names exist on inner sliced elements", {
+  names <- letters[1:5]
+  x <- set_names(1:5, names)
+  exp <- as.list(names)
+  expect_equal(slide_period_vec(x, new_date(1:5), "day", ~list(names(.x))), exp)
+})
+
+test_that("names are never placed on the output", {
+  x <- set_names(1:5, letters[1:5])
+  expect_null(names(slide_period_vec(x, new_date(1:5), "day", ~.x)))
+  expect_null(names(slide_period_vec(x, new_date(1:5), "day", ~.x, .ptype = int())))
+  expect_null(names(slide_period_int(x, new_date(1:5), "day", ~.x)))
+})
