@@ -6,19 +6,30 @@ test_that("size of each `.f` result must be 1", {
     slide_period2_vec(1:2, 1:2, new_date(1:2), "day", ~c(.x, .y)),
     "In iteration 1, the result of `.f` had size 2, not 1"
   )
+  expect_error(
+    slide_period2_int(1:2, 1:2, new_date(1:2), "day", ~c(.x, .y)),
+    "In iteration 1, the result of `.f` had size 2, not 1"
+  )
 })
 
 test_that("inner type is allowed to be different", {
   expect_equal(
-    slide_period2_vec(1:2, 1:2, new_date(1:2), "day", ~if (.x == 1L) {1} else {"hi"}, .ptype = list()),
+    slide_period2_vec(1:2, 1:2, new_date(1:2), "day", ~if (.x == 1L) {list(1)} else {list("hi")}, .ptype = list()),
     list(1, "hi")
   )
 })
 
 test_that("inner type can be restricted with list_of", {
   expect_error(
-    slide_period2_vec(1:2, 1:2, new_date(1:2), "day", ~if (.x == 1L) {1} else {"hi"}, .ptype = list_of(.ptype = double())),
-    class = "vctrs_error_cast_lossy"
+    slide_period2_vec(1:2, 1:2, new_date(1:2), "day", ~if (.x == 1L) {list_of(1)} else {list_of("hi")}, .ptype = list_of(.ptype = double())),
+    class = "vctrs_error_incompatible_type"
+  )
+})
+
+test_that("type can be restricted", {
+  expect_error(
+    slide_period2_int(1:2, 1:2, new_date(1:2), "day", ~if (.x == 1L) {1L} else {"hi"}),
+    class = "vctrs_error_incompatible_type"
   )
 })
 
@@ -28,7 +39,6 @@ test_that("inner type can be restricted with list_of", {
 test_that(".ptype is respected", {
   expect_equal(slide_period2_vec(1, 1, new_date(0), "day", ~.x), 1)
   expect_equal(slide_period2_vec(1, 1, new_date(0), "day", ~.x, .ptype = int()), 1L)
-  expect_equal(slide_period2_vec(1, 1, new_date(0), "day", ~.x, .ptype = new_date()), as.Date("1970-01-02"))
   expect_error(slide_period2_vec(1, 1, new_date(0), "day", ~.x + .5, .ptype = integer()), class = "vctrs_error_cast_lossy")
 })
 
@@ -51,6 +61,10 @@ test_that("`.ptype = NULL` validates that element lengths are 1", {
     slide_period2_vec(1:2, 1:2, new_date(0:1), "day", ~if(.x == 1L) {1:2} else {1}, .ptype = NULL),
     "In iteration 1, the result of `.f` had size 2, not 1."
   )
+  expect_error(
+    slide_period2_vec(1:2, 1:2, new_date(0:1), "day", ~if(.x == 1L) {NULL} else {1}, .ptype = NULL),
+    "In iteration 1, the result of `.f` had size 0, not 1."
+  )
 })
 
 test_that("`.ptype = NULL` returns `NULL` with size 0 `.x`", {
@@ -72,7 +86,9 @@ test_that("with `.complete = TRUE`, `.ptype` is used to pad", {
     ),
     c(NA, 1, 1)
   )
+})
 
+test_that("with `.complete = TRUE`, padding is size stable (#93)", {
   expect_equal(
     slide_period2_vec(
       1:3, 1:3, new_date(1:3),
@@ -80,6 +96,20 @@ test_that("with `.complete = TRUE`, `.ptype` is used to pad", {
     ),
     new_date(c(NA, 0, 0))
   )
+  expect_equal(
+    slide_period2_vec(
+      1:3, 1:3, new_date(1:3),
+      "day", ~new_date(0), .after = 1, .complete = TRUE, .ptype = new_date()
+    ),
+    new_date(c(0, 0, NA))
+  )
+  expect_equal(
+    slide_period2_vec(
+      1:3, 1:3, new_date(1:3),
+      "day", ~new_date(0), .before = 1, .complete = TRUE, .ptype = NULL
+    ),
+    new_date(c(NA, 0, 0))
+  )
 })
 
 test_that("can return a matrix and rowwise bind the results together", {
@@ -90,6 +120,16 @@ test_that("can return a matrix and rowwise bind the results together", {
   )
 })
 
+test_that("`slide_period2_vec()` falls back to `c()` method as required", {
+  local_c_foobar()
+
+  expect_identical(slide_period2_vec(1:3, 1:3, new_date(1:3), "day", ~foobar(.x), .ptype = foobar()), foobar(1:3))
+  expect_condition(slide_period2_vec(1:3, 1:3, new_date(1:3), "day", ~foobar(.x), .ptype = foobar()), class = "slider_c_foobar")
+
+  expect_identical(slide_period2_vec(1:3, 1:3, new_date(1:3), "day", ~foobar(.x)), foobar(1:3))
+  expect_condition(slide_period2_vec(1:3, 1:3, new_date(1:3), "day", ~foobar(.x)), class = "slider_c_foobar")
+})
+
 # ------------------------------------------------------------------------------
 # suffix tests
 
@@ -113,8 +153,8 @@ test_that("slide_period2_chr() works", {
   expect_equal(slide_period2_chr("x", 1, new_date(0), "day", ~.x), "x")
 })
 
-test_that("slide_period2_chr() can coerce", {
-  expect_equal(slide_period2_chr(1, 1, new_date(0), "day", ~.x), "1")
+test_that("slide_period2_chr() cannot coerce", {
+  expect_error(slide_period2_chr(1, 1, new_date(0), "day", ~.x), class = "vctrs_error_incompatible_type")
 })
 
 test_that("slide_period2_lgl() works", {
